/***************** Cabecera de Modulo *************************************
*MODULO: transformadas                                                    *
*PERTENECE AL PROYECTO: CICYT 3D                                          *
*DIRECTORIO:                                                              *
*HISTORIA DE VERSIONES:                                                   *
*     (1.0    14/07/97  LUTFI                                             *
*      .......   .....  .....   ........................                  *
*      .......   .....  .....   ........................                  *
*BIBLIOTECAS EXTERNAS: m (matematica)                                     *
**************************************************************************/


/********************** SECCION DESCRIPCION MODULO ************************
 Este modulo promorciona las transformacio es descomposicion y reconstru-
 cion, tanto estandar como no estandar de una imagen 2D.
 El codigo del calculo de la transformada esta implementado para que los
resultados coincidan con los que proporciona Matlab.
**************************************************************************/


/********************** SECCION DE INCLUDES *******************************
**************************************************************************/
#include "../include/transformadas.h"
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

/********************** SECCION DE MACROS *********************************
**************************************************************************/
#define raiz_de_2 ((TIPO)1.4142135623730950488016887242096980785696718754)
/*#define raiz_de_2 ((TIPO)2.0)*/
#define NORMALIZAR

/********************** SECCION DE VARIABLES GLOBALES *********************
**************************************************************************/


/********************** SECCION DE RESUMEN FUNCIONES **********************
**************************************************************************
Nombre                     Descripcion
======                     ===========
Decomposition              Descompone un vector 
DecompositionStep          c se modifica con la semisuma en los primeros 
                           tamano/2 elementos y la semiresta en los ultimos
                           elementos.
Reconstruction             Reconstruye un vector
ReconstructionStep         se deshace la operacion realizada sobre c 
                           mediante DecompositionStep
StandardDecomposition      Descomposicion estandar de una matriz
StandardReconstruction     Reconstrucion estandar de una  matriz 
NonstandardDecomposition   Descomposicion no estandar de una matriz 
NonstandardReconstruction  Reconstrucion estandar de una  matriz
*/



/********************** SECCION DE DEFINICION FUNCIONES *******************
**************************************************************************/


/******************* Cabecera de Funcion **********************************
*NOMBRE: Decomposition                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descompone un vector                                        *
*PARAMETROS:                                                              *
*   c:        vector a descomponer                                        *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: normalizacion y bucle de llamadas a DecompositionStep         *
**************************************************************************/
int Decomposition (TIPO *c, int tamano) {
  /*int g = tamano*/
  if (tamano <= 0) {
    perror ("Error en Decomposition");
     return -1;
  }
  while (tamano >= 2) {
    DecompositionStep (c, tamano);
    tamano >>= 1; /* g = g/2 */
  }
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: Decomposition2                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descompone un vector                                        *
*PARAMETROS:                                                              *
*   c:        vector a descomponer                                        *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*   etapas:   numero de veces que hay que descomponer                     *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: normalizacion y bucle de llamadas a DecompositionStep         *
**************************************************************************/
int Decomposition2(TIPO *c, int tamano, int etapas) {
  int g = 1<<etapas;
  if (tamano <= 0) {
    perror ("Error en Decomposition");
     return -1;
  }
  while (g >= 2) {
    DecompositionStep (c, g);
    g >>= 1; /* g = g/2 */
  }
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: DecompositionStep                                                *
***************************************************************************
*                                                                         *
*DESCRIPCION:                                                             *
* c se modifica con la semisuma en los primeros tamano/2 elementos y      *
* la semiresta en los ultimos elementos.                                  *
*PARAMETROS:                                                              *
*   c:        vector a descomponer                                        *
*   tamano:   numero de elementos del vector. Tiene que ser multiplo de 2 *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO:                                                               *
**************************************************************************/
int DecompositionStep (TIPO *c, int tamano) {
  int mitadTamano = tamano >> 1;
  int cont;
  
  TIPO *cAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  TIPO *indice_mas_1 = &(c[tamano-1]);
  TIPO *indice = indice_mas_1 - 1;
  
  if ((tamano <= 0) || (!cAux)) {
    perror ("Error en DecompositionStep");
    return -1;
  }
  for (cont = mitadTamano-1; cont >= 0; cont--) {
#ifdef NORMALIZAR
    /* Expresion simplificada. Viene de ((a-b)/2)*raiz_de_2 */
    cAux[cont] = (*indice + *indice_mas_1) / raiz_de_2;
    cAux[mitadTamano + cont] = (*indice - *indice_mas_1) / raiz_de_2;
#else
    cAux[cont] = (*indice + *indice_mas_1)/2;
    cAux[mitadTamano + cont] = (*indice - *indice_mas_1)/2;
#endif
    indice = indice_mas_1-=2;
    indice--;
  }
  memcpy (c, cAux, (size_t) (tamano*sizeof(TIPO)));
  free (cAux);
  cAux=NULL;
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: Reconstruction                                                   *
***************************************************************************
*                                                                         *
*DESCRIPCION: Reconstruye un vector                                       *
*PARAMETROS:                                                              *
*   c:        vector a recomponer                                         *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la  recomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: bucle de llamadas a ReconstructionStep y  normalizacion       *
**************************************************************************/
int Reconstruction (TIPO *c, int tamano) {
  int cont;	
  if (tamano <= 0){
    perror ("Error en Reconstruction");
    return -1;
  }
  for (cont=2; cont <= tamano; cont <<= 1)
    ReconstructionStep (c, cont);
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: Reconstruction2                                                  *
***************************************************************************
*                                                                         *
*DESCRIPCION: Reconstruye un vector                                       *
*PARAMETROS:                                                              *
*   c:        vector a recomponer                                         *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*   etapas:   numero de veces que hay que reconstruir                     *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la  recomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: bucle de llamadas a ReconstructionStep y  normalizacion       *
**************************************************************************/
int Reconstruction2 (TIPO *c, int tamano, int etapas) {
  int cont;
  int g = 1<<etapas;	
  if (tamano <= 0) {
    perror ("Error en Reconstruction");
    return -1;
  }
  for (cont=2; cont <= g; cont <<= 1)
    ReconstructionStep (c, cont);
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: ReconstructionStep                                               *
***************************************************************************
*                                                                         *
*DESCRIPCION:                                                             *
* se deshace la operacion realizada sobre c mediante DecompositionStep    *
*PARAMETROS:                                                              *
*   c:        vector a recomponer                                         *
*   tamano:   numero de elementos del vector.Tiene que ser multiplo de 2  *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la recomposicion se realiza sobre el mismo vector                *
*VAR.GLOBALES:                                                            *
*ALGORITMO:                                                               *
**************************************************************************/
int ReconstructionStep (TIPO *c, int tamano) {
  TIPO *cAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  TIPO *indice = cAux+tamano-1;
  int mitadTamano = tamano >> 1;
  int cont;
  
  if ((tamano <= 0) || (!cAux)){
    perror ("Error en Reconstruction");
    return -1;
  }
  for (cont = mitadTamano - 1; cont >= 0; cont --) {
#ifdef NORMALIZAR
    /* Expresion simplificada. Viene de ((a-b)/2)*raiz_de_2 */
    *(indice--) = (c[cont] - c[mitadTamano + cont]) / raiz_de_2;
    *(indice--) = (c[cont] + c[mitadTamano + cont]) / raiz_de_2;
#else
    *(indice--) = (c[cont] - c[mitadTamano + cont]/2);
    *(indice--) = (c[cont] + c[mitadTamano + cont]/2);
#endif
  }
  memcpy (c, cAux, (size_t) (tamano*sizeof(TIPO)));
  free (cAux);
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardDecomposition                                            *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion estandar de una matriz                       *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada fila se llama a Decomposition y despues por cada     *
*           columna.                                                      *
**************************************************************************/
int StandardDecomposition (TIPO *c, int filas, int columnas) {
  TIPO *columnaAux;
  int cont_filas, cont_colum; 

  columnaAux = (TIPO*) malloc (filas*sizeof(TIPO));
  if ((filas <= 0) || (columnas <= 0) || (columnaAux ==NULL)) {
    perror ("Error en StandardDecomposition");
    return -1;
  }
  for (cont_filas = 0; cont_filas < filas; cont_filas++)
    Decomposition (c+cont_filas*columnas, columnas);
  for (cont_colum = 0; cont_colum < columnas; cont_colum++) {
    for (cont_filas = 0; cont_filas < filas; cont_filas++)
      columnaAux [cont_filas] = c [cont_filas*columnas + cont_colum];
    Decomposition (columnaAux, filas);
    for (cont_filas = 0; cont_filas < filas; cont_filas++)
      c [cont_filas*columnas + cont_colum] = columnaAux[cont_filas];
  }
  free (columnaAux);
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardReconstruction                                           *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna se llama a Reconstruction y despues por cada *
*           fila.                                                         *
**************************************************************************/
int StandardReconstruction (TIPO *c, int filas, int columnas) {
  TIPO *columnaAux;
  int cont_filas, cont_colum;  
  
  columnaAux = (TIPO*) malloc (filas*sizeof(TIPO));
  if ((filas <= 0) || (columnas <= 0) || (columnaAux ==NULL)){
    perror ("Error en StandardReconstruction");
    return -1;
  }
  for (cont_colum = 0; cont_colum < columnas; cont_colum++) {
    for (cont_filas = 0; cont_filas < filas; cont_filas++)
      columnaAux [cont_filas] = c [cont_filas*columnas + cont_colum];
    Reconstruction (columnaAux, filas);
    for (cont_filas = 0; cont_filas < filas; cont_filas++)
      c [cont_filas*columnas + cont_colum] = columnaAux[cont_filas];
  }
  for (cont_filas = 0; cont_filas < filas; cont_filas++)
    Reconstruction (c+cont_filas*columnas, columnas);
  free (columnaAux);	
  return 0;
}


/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardDecomposition2                                           *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion estandar de una matriz                       *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*   etapas:                                                               *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada fila se llama a Decomposition y despues por cada     *
*           columna.                                                      *
**************************************************************************/
int StandardDecomposition2(TIPO *c, int filas, int columnas,
			   int e_filas, int e_columnas) {
  TIPO *columnaAux;
  int cont_filas, cont_colum, cont, g; 
  /*
    char nombre[100];
  */
 
  columnaAux = (TIPO*) malloc (filas*sizeof(TIPO));
  if ((filas <= 0) || (columnas <= 0) || (columnaAux ==NULL)) {
    perror ("Error en StandardDecomposition");
    return -1;
  }
  g = columnas;
  cont = columnas >> (e_columnas-1);
  while (g >= cont) {
    for (cont_filas = 0; cont_filas < filas; cont_filas++)
      DecompositionStep (c+cont_filas*columnas, g);

    g >>= 1; /* g = g/2 */
  }
  g = filas;
  cont = filas >> (e_filas-1);
  while (g >= cont) {
    for (cont_colum = 0; cont_colum < columnas; cont_colum++) {
      for (cont_filas = 0; cont_filas < filas; cont_filas++) {
	columnaAux [cont_filas] = c [cont_filas*columnas + cont_colum];
      }
      DecompositionStep (columnaAux, g);
      for (cont_filas = 0; cont_filas < filas; cont_filas++)
	c [cont_filas*columnas + cont_colum] = columnaAux[cont_filas];
    }

    g >>= 1; /* g = g/2 */
  }
  free (columnaAux);
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardReconstruction2                                          *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna se llama a Reconstruction y despues por cada *
*           fila.                                                         *
**************************************************************************/
int StandardReconstruction2(TIPO *c, int filas, int columnas,
			   int e_filas, int e_columnas) {
  TIPO *columnaAux;
  int cont_filas, cont_colum, g; 
  char nombre[100];
  
  columnaAux = (TIPO*) malloc (filas*sizeof(TIPO));
  if ((filas <= 0) || (columnas <= 0) || (columnaAux ==NULL)) {
    perror ("Error en StandardReconstruction2");
    return -1;
  }
  g = filas>>(e_filas-1);
  while (g <= columnas) {
    for (cont_colum = 0; cont_colum < columnas; cont_colum++) {
      for (cont_filas = 0; cont_filas < filas; cont_filas++)
	columnaAux [cont_filas] = c [cont_filas*columnas + cont_colum];
      ReconstructionStep (columnaAux, g);
      for (cont_filas = 0; cont_filas < filas; cont_filas++) {
	c [cont_filas*columnas + cont_colum] = columnaAux[cont_filas];
      }
    }
    sprintf (nombre, "StdRecon2-C%03i.pgm", g);
    Escalar (c, filas, columnas, nombre);
    g <<= 1; /* g = g/2 */
  }
  g = columnas>>(e_columnas-1);
  while (g <= filas) {
    for (cont_filas = 0; cont_filas < filas; cont_filas++) {
      ReconstructionStep (c+cont_filas*columnas, g);
    }
    sprintf (nombre, "StdRecon2-F%03i.pgm", g);
    Escalar (c, filas, columnas, nombre);
    g <<= 1; /* g = g/2 */
  }
  free (columnaAux);
  return 0;
}


/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardDecomposition                                         *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz                    *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*           a Decomposition                                               *
**************************************************************************/
int NonstandardDecomposition (TIPO *c, int tamano) {
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;
  char nombre[100];

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return -1;
  }
  while (g >= 2) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    /*    pf = fopen (nombre, "w");
	  for (cont_filas=0; cont_filas < tamano; cont_filas++) {
	  for (cont_colum=0; cont_colum < tamano; cont_colum++)
	  fprintf (pf, "%f ", c[cont_filas*tamano+cont_colum]);
	  fprintf (pf, "\n");
	  
	  }
	  fclose (pf);
    */
    g >>= 1;
  }
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardReconstruction                                        *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna y fila se llama a Reconstruction y despues   *
*           se normaliza.                                                 *
**************************************************************************/
int NonstandardReconstruction (TIPO *c, int tamano) {
  int cont_filas, cont_colum;
  int g = 2;
  TIPO *columnaAux; 

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardReconstruction");
    return -1;
  }
  while (g <= tamano) {
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      ReconstructionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }
    for (cont_filas=0; cont_filas < g; cont_filas++)
      ReconstructionStep (c + cont_filas*tamano, g);
    g <<= 1;
  }
  return 0;
}



/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardDecomposition2                                        *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz                    *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene que ser      *
*             potencia de 2                                               *
*   p:        Indica el numero de pasos que se aplicaran a la descompos.  *
*             Su valor debe ser 2^Numero_de_pasos.                        * 
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*           a Decomposition                                               *
**************************************************************************/
int NonstandardDecomposition2(TIPO *c, int tamano, int p) {
  int cont_filas, cont_colum;
  int g = 1 << p;
  TIPO *columnaAux;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return -1;
  }
  while (g >= 2) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }
    g >>= 1;
  }
  return 0;
}

/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardReconstruction2                                       *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*   p:        Indica el numero de pasos que se aplicaran a la recompos.   *
*             Su valor debe ser 2^Numero_de_pasos.                        * 
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna y fila se llama a Reconstruction y despues   *
*           se normaliza.                                                 *
**************************************************************************/
int NonstandardReconstruction2(TIPO *c, int tamano, int p) {
  int cont_filas, cont_colum;
  int g = 2;
  TIPO *columnaAux; 

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardReconstruction");
    return -1;
  }
  while (g <= 1<<p) {
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      ReconstructionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }
    for (cont_filas=0; cont_filas < g; cont_filas++)
      ReconstructionStep (c + cont_filas*tamano, g);
    g <<= 1;
  }
  return 0;
}

void Escalar (TIPO *c, int filas, int columnas, char *nombre) {
  int cont;
  int i,j;
  FILE *ffsal;
  TIPO maximo, minimo, valor, factor;
  unsigned char *cAux = (unsigned char*)malloc (filas*columnas*sizeof(unsigned char));

  /* Verificamos que el malloc funciono bien */
  if (cAux == NULL)
    {
      perror("Escalar");
      exit(1);
    }

  maximo = minimo = *c;
  for (cont = 0; cont < filas*columnas; cont++) {
    valor = c [cont];
    if (valor < minimo)
      minimo = valor;
    if (valor > maximo)
      maximo = valor;
  }
  factor = 255.0 / (maximo - minimo);
  for (cont = 0; cont < filas*columnas; cont++)
    cAux [cont] = ((c[cont] - minimo) * factor);


  printf("Escribiendo %s %f %f\n",nombre, minimo, maximo);
  
  ffsal=fopen(nombre,"wb");
  if (ffsal == NULL) 
    { 
      printf("Error en la creacion de %s\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }
  /* escribimos la cabecera */
  fprintf(ffsal, "P5\n%d %d\n255\n", columnas, filas);
  
  /* Escritura de la imagen en formato RAW */
  for (j=0;j<filas;j++)
    {
      for (i=0;i<columnas;i++)
      {
	/* Aqui se realiza la conversion */
	if(fputc(cAux[columnas*j+i], ffsal) == EOF)
	  { 
	    printf("Error en la escritura de %s i: %d j: %d\n",nombre,i,j);
	    printf("Error: %d  %s\n",errno,strerror(errno));
	    exit(1);
	  }
      }
    }
  fclose(ffsal);
  free (cAux);
}
